@@ -36,7 +36,7 @@ def _resolve_implicit_col_feature_mappings_str(
 
 def query_arrow_table(query: str, table: pa.Table) -> pa.Table:
     if duckdb is None or sqlglot is None:
-        raise missing_dependency_exception("chalkpy[sql]")
+        raise missing_dependency_exception("chalkpy[runtime]")
     t: sqlglot.exp.Table
     table_names = list({t.name for t in sqlglot.parse_one(query).find_all(sqlglot.exp.Table)})
     con = duckdb.connect(database=":memory:")
@@ -56,7 +56,7 @@ def query_as_features(
     try:
         import polars as pl
     except ImportError:
-        raise missing_dependency_exception("chalkpy[polars]")
+        raise missing_dependency_exception("chalkpy[runtime]")
     res = query_arrow_table(query, table)
     col_feature_mappings = _resolve_implicit_col_feature_mappings_str(
         # Note: snowflake by default capitalizes all table and column names, although it's
@@ -84,7 +84,7 @@ def query_as_feature_formatted(
     try:
         import polars as pl
     except ImportError:
-        raise missing_dependency_exception("chalkpy[polars]")
+        raise missing_dependency_exception("chalkpy[runtime]")
 
     unformatted = formatted_query.replace(str(table), "__chalk_table__")
     pa_table = table
