@@ -5,7 +5,6 @@ from chalk._monitoring.Chart import (
     Chart,
     ComparatorKind,
     FilterKind,
-    Formula,
     GroupByKind,
     MetricFilter,
     MetricFormulaKind,
@@ -15,6 +14,7 @@ from chalk._monitoring.Chart import (
     Trigger,
     WindowFunctionKind,
     _DatasetFeatureOperand,
+    _Formula,
     _MultiSeriesOperand,
     _SingleSeriesOperand,
 )
@@ -79,7 +79,7 @@ def _convert_group_by(group_by: GroupByKind) -> GroupByKindGQL:
     return GroupByKindGQL(group_by.value.upper())
 
 
-def _convert_formula(formula: Formula) -> CreateMetricFormulaGQL:
+def _convert_formula(formula: _Formula) -> CreateMetricFormulaGQL:
     operands = formula._operands
     if not operands:
         raise ValueError("'operands' is a required value for Formula instances")
