@@ -2,7 +2,12 @@ from typing import Optional
 
 from chalk.client.client_impl import ChalkAPIClientImpl
 from chalk.client.client_protocol import ChalkAPIClientProtocol, OnlineQueryResponseProtocol
-from chalk.client.exc import ChalkBaseException
+from chalk.client.exc import (
+    ChalkBaseException,
+    ChalkDatasetDownloadException,
+    ChalkOfflineQueryException,
+    ChalkResolverRunException,
+)
 from chalk.client.models import (
     ChalkError,
     ChalkException,
@@ -18,6 +23,9 @@ from chalk.client.models import (
 
 __all__ = [
     "ChalkError",
+    "ChalkResolverRunException",
+    "ChalkDatasetDownloadException",
+    "ChalkOfflineQueryException",
     "FeatureResult",
     "WhoAmIResponse",
     "OnlineQueryResponse",
