@@ -2,27 +2,35 @@ from __future__ import annotations
 
 import collections.abc
 import time
+import uuid
 from datetime import datetime, timezone
-from typing import TYPE_CHECKING, Any, List, Mapping, Optional, Sequence, Type, TypeVar, Union
+from typing import TYPE_CHECKING, Any, List, Mapping, Optional, Sequence, Type, TypeVar, Union, cast
 from urllib.parse import urljoin
 
 import pandas as pd
 import requests
 from pydantic import BaseModel, ValidationError
 from requests import HTTPError
+from requests import JSONDecodeError as RequestsJSONDecodeError
 
 from chalk._version import __version__ as chalkpy_version
 from chalk.client.client_protocol import ChalkAPIClientProtocol, OnlineQueryResponseProtocol
 from chalk.client.dataset import DatasetVersion, load_dataset
-from chalk.client.exc import ChalkOfflineQueryException, ChalkResolverRunException
+from chalk.client.exc import (
+    ChalkBaseException,
+    ChalkComputeResolverException,
+    ChalkOfflineQueryException,
+    ChalkOnlineQueryException,
+    ChalkResolverRunException,
+    ChalkWhoAmIException,
+)
 from chalk.client.models import (
     ChalkError,
     ComputeResolverOutputRequest,
     ComputeResolverOutputResponse,
     CreateOfflineQueryJobRequest,
     CreateOfflineQueryJobResponse,
-    ExchangeCredentialsRequest,
-    ExchangeCredentialsResponse,
+    ErrorCode,
     FeatureResult,
     GetOfflineQueryJobResponse,
     OfflineQueryContext,
@@ -31,13 +39,14 @@ from chalk.client.models import (
     OnlineQueryRequest,
     OnlineQueryResponse,
     ResolverRunResponse,
-    TriggerResolverRunRequest,
     WhoAmIResponse,
+    _ExchangeCredentialsRequest,
+    _ExchangeCredentialsResponse,
+    _TriggerResolverRunRequest,
 )
 from chalk.config.auth_config import load_token
-from chalk.features import DataFrame, Feature, FeatureWrapper, ensure_feature
+from chalk.features import DataFrame, Feature, FeatureNotFoundException, ensure_feature
 from chalk.features.pseudofeatures import CHALK_TS_FEATURE
-from chalk.serialization.codec import FEATURE_CODEC
 from chalk.utils.log_with_context import get_logger
 from chalk.utils.missing_dependency import missing_dependency_exception
 
@@ -68,11 +77,13 @@ class OnlineQueryResponseImpl(OnlineQueryResponseProtocol):
         for d in self.data:
             if d.value is not None:
                 try:
-                    d.value = FEATURE_CODEC.decode_fqn(d.field, d.value)
-                except Exception:
+                    f = Feature.from_root_fqn(d.field)
+                except FeatureNotFoundException:
                     self.warnings.append(
                         f"Return data {d.field}:{d.value} cannot be decoded. Attempting to JSON decode"
                     )
+                else:
+                    d.value = f.converter.from_json_to_rich(d.value)
 
         self._values = {d.field: d for d in self.data}
 
@@ -151,7 +162,7 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
                 "Accept": "application/json",
                 "Content-Type": "application/json",
             },
-            json=ExchangeCredentialsRequest(
+            json=_ExchangeCredentialsRequest(
                 client_id=self._config.client_id,
                 client_secret=self._config.client_secret,
                 grant_type="client_credentials",
@@ -161,7 +172,7 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
         resp.raise_for_status()
         response_json = resp.json()
         try:
-            creds = ExchangeCredentialsResponse(**response_json)
+            creds = _ExchangeCredentialsResponse(**response_json)
         except ValidationError:
             raise HTTPError(response=resp)
         self._default_headers["Authorization"] = f"Bearer {creds.access_token}"
@@ -175,6 +186,36 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
             headers["X-Chalk-Env-Id"] = x_chalk_env_id
         return headers
 
+    @staticmethod
+    def _raise_if_200_with_errors(response: BaseModel, exception_cls: Type[ChalkBaseException]):
+        errors = getattr(response, "errors", None)
+        if errors and isinstance(errors, list) and all(isinstance(e, ChalkError) for e in errors):
+            errors = cast(List[ChalkError], errors)
+            raise exception_cls(errors=errors)
+
+    @staticmethod
+    def _raise_http_error(http_error: HTTPError, exception_cls: Type[ChalkBaseException]):
+        detail = None
+        try:
+            response_json = http_error.response.json()
+            if isinstance(response_json, Mapping):
+                detail = response_json.get("detail")
+        except RequestsJSONDecodeError:
+            pass
+
+        status_code = http_error.response.status_code
+        known_error_code = None
+        if status_code == 401:
+            known_error_code = ErrorCode.UNAUTHENTICATED
+        elif status_code == 403:
+            known_error_code = ErrorCode.UNAUTHORIZED
+
+        chalk_error = ChalkError(
+            code=known_error_code or ErrorCode.INTERNAL_SERVER_ERROR,
+            message=detail or f"Unexpected Chalk server error with status code {status_code}",
+        )
+        raise exception_cls(errors=[chalk_error])
+
     def _request(
         self,
         method: str,
@@ -183,12 +224,16 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
         json: Optional[BaseModel],
         use_engine: bool,
         environment_override: Optional[str],
+        exception_cls: Type[ChalkBaseException],
     ) -> T:
         # Track whether we already exchanged credentials for this request
         exchanged_credentials = False
         if not self._exchanged_credentials:
             exchanged_credentials = True
-            self._exchange_credentials()
+            try:
+                self._exchange_credentials()
+            except HTTPError as e:
+                self._raise_http_error(http_error=e, exception_cls=exception_cls)
         headers = self._get_headers(environment_override=environment_override)
         url = urljoin(self._config.api_server, uri)
         json_body = json and json.dict()
@@ -199,8 +244,14 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
             self._exchange_credentials()
             r = requests.request(method=method, headers=headers, url=url, json=json_body)
 
-        r.raise_for_status()
-        return response(**r.json())
+        try:
+            r.raise_for_status()
+        except HTTPError as e:
+            self._raise_http_error(http_error=e, exception_cls=exception_cls)
+
+        response = response(**r.json())
+
+        return response
 
     def whoami(self) -> WhoAmIResponse:
         return self._request(
@@ -210,11 +261,12 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
             json=None,
             use_engine=False,
             environment_override=None,
+            exception_cls=ChalkWhoAmIException,
         )
 
     def upload_features(
         self,
-        input: Mapping[Union[str, Feature, FeatureWrapper, Any], Any],
+        input: Mapping[Union[str, Feature, Any], Any],
         context: Optional[OnlineQueryContext] = None,
         preview_deployment_id: Optional[str] = None,
         correlation_id: Optional[str] = None,
@@ -234,9 +286,9 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
 
     def query(
         self,
-        input: Mapping[Union[str, Feature, FeatureWrapper, Any], Any],
-        output: Sequence[Union[str, Feature, FeatureWrapper, Any]],
-        staleness: Optional[Mapping[Union[str, Feature, FeatureWrapper, Any], str]] = None,
+        input: Mapping[Union[str, Feature, Any], Any],
+        output: Sequence[Union[str, Feature, Any]],
+        staleness: Optional[Mapping[Union[str, Feature, Any], str]] = None,
         context: Optional[OnlineQueryContext] = None,
         preview_deployment_id: Optional[str] = None,
         correlation_id: Optional[str] = None,
@@ -248,10 +300,15 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
         for feature, value in input.items():
             try:
                 feature = ensure_feature(feature)
-                encoded_inputs[feature.fqn] = FEATURE_CODEC.encode(feature, value)
-            except Exception:
+            except FeatureNotFoundException:
                 warnings.append(f"Input {feature} not recognized. JSON encoding {feature} and requesting anyways")
                 encoded_inputs[feature] = value
+            else:
+                encoded_inputs[feature.fqn] = feature.converter.from_rich_to_json(
+                    value,
+                    missing_value_strategy="error",
+                )
+
         outputs: List[str] = [str(feature) for feature in output]
 
         request = OnlineQueryRequest(
@@ -272,15 +329,16 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
             response=OnlineQueryResponse,
             use_engine=preview_deployment_id is None,
             environment_override=context.environment if context else None,
+            exception_cls=ChalkOnlineQueryException,
         )
         return OnlineQueryResponseImpl(data=resp.data, errors=resp.errors or [], warnings=warnings)
 
     def get_training_dataframe(
         self,
-        input: Union[Mapping[Union[str, Feature, FeatureWrapper, Any], Any], pd.DataFrame, pl.DataFrame, DataFrame],
+        input: Union[Mapping[Union[str, Feature, Any], Any], pd.DataFrame, pl.DataFrame, DataFrame],
         input_times: Union[Sequence[datetime], datetime, None] = None,
-        output: Sequence[Union[str, Feature, FeatureWrapper, Any]] = (),
-        required_output: Sequence[Union[str, Feature, FeatureWrapper, Any]] = (),
+        output: Sequence[Union[str, Feature, Any]] = (),
+        required_output: Sequence[Union[str, Feature, Any]] = (),
         output_ts: bool = False,
         context: Optional[OfflineQueryContext] = None,
         dataset: Optional[str] = None,
@@ -293,7 +351,7 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
         try:
             import polars as pl
         except ImportError:
-            raise missing_dependency_exception("chalkpy[polars]")
+            raise missing_dependency_exception("chalkpy[runtime]")
         if isinstance(input, (DataFrame, pl.DataFrame)):
             input = input.to_pandas()
 
@@ -349,12 +407,16 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
         for col_index, column in enumerate(matrix):
             for row_index, value in enumerate(column):
                 try:
-                    encoded_feature = FEATURE_CODEC.encode_fqn(
-                        fqn=columns_fqn[col_index],
-                        value=value,
-                    )
-                except (KeyError, ValueError, TypeError):
+                    f = Feature.from_root_fqn(columns_fqn[col_index])
+                except FeatureNotFoundException:
+                    # The feature is not in the graph, so passing the value as-is and hoping it's possible
+                    # to json-serialize it
                     encoded_feature = value
+                else:
+                    encoded_feature = f.converter.from_rich_to_json(
+                        value,
+                        missing_value_strategy="error",
+                    )
 
                 matrix[col_index][row_index] = encoded_feature
 
@@ -379,8 +441,8 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
 
     def sample(
         self,
-        output: Sequence[Union[str, Feature, FeatureWrapper, Any]],
-        required_output: Sequence[Union[str, Feature, FeatureWrapper, Any]] = (),
+        output: Sequence[Union[str, Feature, Any]] = (),
+        required_output: Sequence[Union[str, Feature, Any]] = (),
         output_id: bool = False,
         output_ts: bool = False,
         max_samples: Optional[int] = None,
@@ -397,22 +459,18 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
         if len(output) == 0 and len(required_output) == 0:
             raise ValueError("Either 'output' or 'required_output' must be specified.")
 
-        try:
-            response = self._create_and_await_offline_query_job(
-                query_input=None,
-                optional_output=optional_output_root_fqns,
-                required_output=required_output_root_fqns,
-                max_samples=max_samples,
-                context=context,
-                output_id=output_id,
-                output_ts=output_ts,
-                dataset_name=dataset,
-                branch=branch,
-                disallow_stale_results=disallow_stale_results,
-            )
-        except HTTPError as e:
-            _logger.debug("Got HTTP Exception while processing sample query", exc_info=e)
-            raise ChalkOfflineQueryException(message=f"HTTP error while processing sample query.", errors=[])
+        response = self._create_and_await_offline_query_job(
+            query_input=None,
+            optional_output=optional_output_root_fqns,
+            required_output=required_output_root_fqns,
+            max_samples=max_samples,
+            context=context,
+            output_id=output_id,
+            output_ts=output_ts,
+            dataset_name=dataset,
+            branch=branch,
+            disallow_stale_results=disallow_stale_results,
+        )
 
         return response.to_pandas()
 
@@ -425,45 +483,32 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
         if context is None:
             context = OfflineQueryContext()
 
-        request = TriggerResolverRunRequest(resolver_fqn=resolver_fqn)
-        try:
-            response = self._request(
-                method="POST",
-                uri="/v1/runs/trigger",
-                json=request,
-                response=ResolverRunResponse,
-                use_engine=False,
-                environment_override=context.environment,
-            )
-        except HTTPError as e:
-
-            detail = e.response.json().get("detail")
-            if detail is not None:
-                raise ChalkResolverRunException(message=detail)
-            else:
-                raise
+        request = _TriggerResolverRunRequest(resolver_fqn=resolver_fqn)
+        response = self._request(
+            method="POST",
+            uri="/v1/runs/trigger",
+            json=request,
+            response=ResolverRunResponse,
+            use_engine=False,
+            environment_override=context.environment,
+            exception_cls=ChalkResolverRunException,
+        )
 
         return response
 
     def get_run_status(self, run_id: str, context: Optional[OfflineQueryContext] = None) -> ResolverRunResponse:
         if context is None:
             context = OfflineQueryContext()
-        try:
-            response = self._request(
-                method="GET",
-                uri=f"/v1/runs/{run_id}",
-                response=ResolverRunResponse,
-                json=None,
-                use_engine=False,
-                environment_override=context.environment,
-            )
-        except HTTPError as e:
 
-            detail = e.response.json().get("detail")
-            if detail is not None:
-                raise ChalkResolverRunException(message=detail)
-            else:
-                raise
+        response = self._request(
+            method="GET",
+            uri=f"/v1/runs/{run_id}",
+            response=ResolverRunResponse,
+            json=None,
+            use_engine=False,
+            environment_override=context.environment,
+            exception_cls=ChalkResolverRunException,
+        )
 
         return response
 
@@ -491,9 +536,7 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
             max_cache_age_secs=0 if disallow_stale_results else None,
         )
         response = self._create_offline_query_job(request=req, context=context)
-
-        if response.errors is not None and len(response.errors) > 0:
-            raise ChalkOfflineQueryException(message="Failed to execute offline query", errors=response.errors)
+        self._raise_if_200_with_errors(response=response, exception_cls=ChalkOfflineQueryException)
 
         while True:
             status = self._get_job_status(job_id=response.job_id, context=context)
@@ -530,10 +573,15 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
 
         for col_index, column in enumerate(matrix):
             for row_index, value in enumerate(column):
-                matrix[col_index][row_index] = FEATURE_CODEC.encode_fqn(
-                    fqn=columns_fqn[col_index],
-                    value=value,
-                )
+                try:
+                    f = Feature.from_root_fqn(columns_fqn[col_index])
+                except FeatureNotFoundException:
+                    # The feature is not in the graph, so passing the value as-is and hoping it's possible
+                    # to json-serialize it
+                    encoded_feature = value
+                else:
+                    encoded_feature = f.converter.from_rich_to_json(value, missing_value_strategy="error")
+                matrix[col_index][row_index] = encoded_feature
 
         query_input = OfflineQueryInput(
             columns=columns_fqn,
@@ -547,10 +595,9 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
             response=ComputeResolverOutputResponse,
             environment_override=context.environment,
             use_engine=False,
+            exception_cls=ChalkComputeResolverException,
         )
-
-        if response.errors is not None and len(response.errors) > 0:
-            raise ChalkOfflineQueryException(message="Failed to execute offline query", errors=response.errors)
+        self._raise_if_200_with_errors(response=response, exception_cls=ChalkComputeResolverException)
 
         while True:
             status = self._get_compute_job_status(job_id=response.job_id, context=context)
@@ -568,6 +615,7 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
             json=None,
             use_engine=False,
             environment_override=context.environment,
+            exception_cls=ChalkComputeResolverException,
         )
 
     def _create_offline_query_job(self, request: CreateOfflineQueryJobRequest, context: OfflineQueryContext):
@@ -578,10 +626,11 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
             response=CreateOfflineQueryJobResponse,
             environment_override=context.environment,
             use_engine=False,
+            exception_cls=ChalkOfflineQueryException,
         )
         return response
 
-    def _get_job_status(self, job_id: str, context: OfflineQueryContext) -> GetOfflineQueryJobResponse:
+    def _get_job_status(self, job_id: uuid.UUID, context: OfflineQueryContext) -> GetOfflineQueryJobResponse:
         return self._request(
             method="GET",
             uri=f"/v2/offline_query/{job_id}",
@@ -589,4 +638,5 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
             environment_override=context.environment,
             use_engine=False,
             json=None,
+            exception_cls=ChalkOfflineQueryException,
         )
