@@ -2,8 +2,9 @@ import inspect
 import logging
 from typing import Any, Callable, Iterable, List, Optional, Set, Union, overload
 
+from chalk.features.tag import Environments
+
 HookFn = Callable[[], Any]
-EnvironmentFilter = Optional[Union[List[str], str]]
 
 
 def _run_all_hooks(environment: str, hooks: Iterable["Hook"]) -> None:
@@ -25,7 +26,7 @@ class Hook:
     fn: HookFn
     filename: str
 
-    def __init__(self, fn: HookFn, filename: str, environment: Optional[EnvironmentFilter] = None):
+    def __init__(self, fn: HookFn, filename: str, environment: Optional[Environments] = None):
         self.fn = fn
         self.filename = filename
         self.environment = (
@@ -57,12 +58,12 @@ def before_all(fn: HookFn, /) -> Hook:
 
 
 @overload
-def before_all(fn: None = None, /, environment: EnvironmentFilter = None) -> Callable[[HookFn], Hook]:
+def before_all(fn: None = None, /, environment: Environments = None) -> Callable[[HookFn], Hook]:
     ...
 
 
 def before_all(
-    fn: Optional[HookFn] = None, /, environment: EnvironmentFilter = None
+    fn: Optional[HookFn] = None, /, environment: Environments = None
 ) -> Union[Hook, Callable[[HookFn], Hook]]:
     caller_filename = inspect.stack()[1].filename
 
@@ -75,17 +76,17 @@ def before_all(
 
 
 @overload
-def after_all(fn: HookFn, /, environment: EnvironmentFilter = None) -> Hook:
+def after_all(fn: HookFn, /, environment: Environments = None) -> Hook:
     ...
 
 
 @overload
-def after_all(fn: None = None, /, environment: EnvironmentFilter = None) -> Callable[[HookFn], Hook]:
+def after_all(fn: None = None, /, environment: Environments = None) -> Callable[[HookFn], Hook]:
     ...
 
 
 def after_all(
-    fn: Optional[HookFn] = None, /, environment: EnvironmentFilter = None
+    fn: Optional[HookFn] = None, /, environment: Environments = None
 ) -> Union[Hook, Callable[[HookFn], Hook]]:
     caller_filename = inspect.stack()[1].filename
 
