@@ -47,5 +47,15 @@ def is_primary(f: Any) -> bool:
     -------
     bool
         `True` if `f` is primary and `False` otherwise.
+
+    Examples
+    --------
+    >>> from chalk.features import features
+    >>> @features
+    ... class User:
+    ...     uid: Primary[int]
+    ...     email: str
+    >>> assert is_primary(User.uid)
+    >>> assert not is_primary(User.email)
     """
     return unwrap_feature(f).primary
