@@ -1,4 +1,5 @@
 from datetime import datetime
+from typing import List
 
 from chalk.features.feature_field import Feature
 
@@ -12,7 +13,6 @@ __all__ = [
 ]
 
 PSEUDONAMESPACE = "__chalk__"
-
 CHALK_TS_FEATURE = Feature(
     name="CHALK_TS",
     namespace=PSEUDONAMESPACE,
@@ -32,4 +32,4 @@ REPLACED_OBSERVED_AT_FEATURE = Feature(
     etl_offline_to_online=False,
 )
 
-PSEUDOFEATURES = [CHALK_TS_FEATURE, ID_FEATURE, OBSERVED_AT_FEATURE, REPLACED_OBSERVED_AT_FEATURE]
+PSEUDOFEATURES: List[Feature] = [CHALK_TS_FEATURE, ID_FEATURE, OBSERVED_AT_FEATURE, REPLACED_OBSERVED_AT_FEATURE]
