@@ -22,7 +22,7 @@ def _handle_negation(file_path: Union[str, Path], rules: List[IgnoreRule]):
 def parse_gitignore(full_path: Union[str, Path], base_dir: Optional[str] = None) -> Callable[[Union[str, Path]], bool]:
     if base_dir is None:
         base_dir = dirname(full_path)
-    rules = []
+    rules: List[IgnoreRule] = []
     with open(full_path) as ignore_file:
         counter = 0
         for line in ignore_file:
@@ -32,7 +32,7 @@ def parse_gitignore(full_path: Union[str, Path], base_dir: Optional[str] = None)
             if rule:
                 rules.append(rule)
     if not any(r.negation for r in rules):
-        return lambda file_path: any(r.match(file_path) for r in rules)
+        return lambda file_path: any(r.match(str(file_path)) for r in rules)
     else:
         # We have negation rules. We can't use a simple "any" to evaluate them.
         # Later rules override earlier rules.
@@ -110,7 +110,8 @@ def _rule_from_pattern(pattern: str, base_path: Optional[Path] = None, source: O
         i = i - 1
     regex = _fnmatch_pathname_to_regex(pattern, directory_only)
     if anchored:
-        regex = "".join(["^", regex])
+        regex = f"^{regex}"
+    regex = f"(?ms){regex}"
     return IgnoreRule(
         pattern=orig_pattern,
         regex=regex,
@@ -208,7 +209,6 @@ def _fnmatch_pathname_to_regex(pattern: str, directory_only: bool):
                 res.append("[{}]".format(stuff))
         else:
             res.append(re.escape(c))
-    res.insert(0, "(?ms)")
     if not directory_only:
         res.append("$")
     return "".join(res)
