@@ -26,6 +26,7 @@ class MySQLSourceImpl(BaseSQLSource, TableIngestMixIn, SQLSourceWithTableIngestP
         except ModuleNotFoundError:
             raise missing_dependency_exception("chalkpy[mysql]")
         del pymysql
+        self.name = name
         self.host = host or load_integration_variable(name="MYSQL_HOST", integration_name=name)
         self.port = (
             int(port)
@@ -39,11 +40,14 @@ class MySQLSourceImpl(BaseSQLSource, TableIngestMixIn, SQLSourceWithTableIngestP
         BaseSQLSource.__init__(self, name=name)
 
     def engine_args(self) -> Mapping[str, Any]:
-        return dict(
+        default = dict(
             pool_size=20,
             max_overflow=60,
             pool_pre_ping=True,
         )
+        additional = self.load_env_engine_args(name=self.name)
+        default.update(additional)
+        return default
 
     def local_engine_url(self) -> URL:
         from sqlalchemy.engine.url import URL
