@@ -67,6 +67,7 @@ class PostgreSQLSourceImpl(BaseSQLSource, TableIngestMixIn, SQLSourceWithTableIn
         except ImportError:
             raise missing_dependency_exception("chalkpy[postgresql]")
         del psycopg2  # unused
+        self.name = name
         self.host = host or load_integration_variable(integration_name=name, name="PGHOST")
         self.port = (
             int(port)
@@ -80,7 +81,7 @@ class PostgreSQLSourceImpl(BaseSQLSource, TableIngestMixIn, SQLSourceWithTableIn
         BaseSQLSource.__init__(self, name=name)
 
     def engine_args(self) -> Mapping[str, Any]:
-        return dict(
+        default = dict(
             pool_size=20,
             max_overflow=60,
             pool_pre_ping=True,
@@ -92,6 +93,9 @@ class PostgreSQLSourceImpl(BaseSQLSource, TableIngestMixIn, SQLSourceWithTableIn
                 "keepalives_count": 5,
             },
         )
+        additional = self.load_env_engine_args(name=self.name)
+        default.update(additional)
+        return default
 
     def local_engine_url(self) -> URL:
         from sqlalchemy.engine.url import URL
